#!/bin/zsh

# Copyright 2014, 2015 Paul Hänsch
#
# This file is part of shcgi.
# 
# shcgi is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# shcgi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with shcgi.  If not, see <http://www.gnu.org/licenses/>. 

export LC_ALL=C LANG=C
DBG_ENABLED=false
DBG_FILE=/dev/stderr

# this program is supposed to be symlinked into a http root directory
# we will use the http root as object storage (data directory) and call sub
# programs from the directory in which the real executable resides
# therefore we need to identify the code and data directories _EXEC and _DATA
call="$0"
real="$(readlink -f $call)"
_DATA="$(dirname "$call")"  #storage directory
_EXEC="${real%/shcgi/index.cgi}"  #execution directory

# put debug options in the local.opts file
[ -r "$_DATA/local.opts" ] && . "$_DATA/local.opts"

# basic functions
debug() { #change to false to disable debugging
  if [ "$DBG_ENABLED" = true -a -n "$*" ]; then
    printf '%s\n' "$*" >>"$DBG_FILE"
  elif [ "$DBG_ENABLED" = true -a -z "$*" ]; then
    tee -a "$DBG_FILE"
  elif [ -z "$*" ]; then
    cat
  fi
}

die() {
  debug FATAL: "$*"
  printf '%s\n' "$*" >&2
  exit 1
}

data_dirs(){
  # create directories for object storage

  [ -d "$_DATA" -a -w "$_DATA" ] || die "storage directory must be writable"
  for each in "$@"; do
    [ ! -e "$_DATA/$each" ] && mkdir "$_DATA/$each"
    [ -d "$_DATA/$each" -a -w "$_DATA/$each" ] || die "storage \"$_DATA/$each\" must be a writable directory"
  done
}

# create htaccess file
[ -f "$_DATA/.htaccess" ] || cat >"$_DATA/.htaccess" <<EOF
Options		+ExecCGI
AddHandler	cgi-script .cgi
DirectoryIndex	index.cgi
EOF
[ -f "$_DATA/.htaccess" ] || die "no htaccess file present and unable to create one"

. "$_EXEC/shcgi/cgi.sh"

cgi_get

[ -x "$_EXEC/constants.sh" ] && . "$_EXEC/constants.sh"

if [ -n "$_GET[\"action\"]" ]; then
  . "$_EXEC/shcgi/action.sh"
else
  . "$_EXEC/shcgi/page.sh"
fi
